"""
test_django-watchman
------------

Tests for `django-watchman` decorators module.
"""

import logging
import unittest
from typing import cast
from unittest import mock
from unittest.mock import MagicMock, patch

from django.test.client import Client
from django.urls import reverse

from watchman import settings as watchman_settings
from watchman.decorators import check, parse_auth_header
from watchman.types import CheckStatus


class TestParseAuthHeader(unittest.TestCase):
    def test_quoted_token(self):
        self.assertEqual(
            parse_auth_header('WATCHMAN-TOKEN Token="ABC123"'),
            "ABC123",
        )

    def test_unquoted_token(self):
        self.assertEqual(
            parse_auth_header("WATCHMAN-TOKEN Token=ABC123"),
            "ABC123",
        )

    def test_token_with_dashes(self):
        self.assertEqual(
            parse_auth_header('WATCHMAN-TOKEN Token="123-456-ABCD"'),
            "123-456-ABCD",
        )

    def test_unquoted_token_with_dashes(self):
        self.assertEqual(
            parse_auth_header("WATCHMAN-TOKEN Token=123-456-ABCD"),
            "123-456-ABCD",
        )

    def test_missing_token_key_raises_key_error(self):
        with self.assertRaises(KeyError):
            parse_auth_header("WATCHMAN-TOKEN NotToken=ABC123")

    def test_empty_string_raises_key_error(self):
        with self.assertRaises(KeyError):
            parse_auth_header("")

    def test_no_equals_raises_key_error(self):
        with self.assertRaises(KeyError):
            parse_auth_header("WATCHMAN-TOKEN NoEqualsHere")

    def test_multiple_params_extracts_token(self):
        self.assertEqual(
            parse_auth_header('WATCHMAN-TOKEN Foo="bar" Token="SECRET"'),
            "SECRET",
        )


class FakeException(Exception):
    def __init__(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)


class TestDBConnection(unittest.TestCase):
    def setUp(self):
        self.client = Client()

    @patch("watchman.checks.connections")
    def test_cursor_is_called(self, mock_connections):
        cursor_mock = MagicMock()
        mock_connections["default"].cursor().__enter__.return_value = cursor_mock
        data = {
            "watchman-token": "t1",
        }
        self.client.get(reverse("status"), data)
        cursor_mock.execute.assert_called_once_with("SELECT 1")


class TestWatchmanMultiTokens(unittest.TestCase):
    def setUp(self):
        self.client = Client()
        watchman_settings.WATCHMAN_TOKEN = None
        watchman_settings.WATCHMAN_TOKENS = "t1,t2"

    def test_200_ok_if_matching_first_token_in_list(self):
        data = {
            "watchman-token": "t1",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 200)

    def test_200_ok_if_matching_second_token_in_list(self):
        data = {
            "watchman-token": "t2",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 200)

    def test_403_raised_if_missing_token(self):
        response = self.client.get(reverse("status"))
        self.assertEqual(response.status_code, 403)

    def test_403_raised_if_invalid_token(self):
        data = {
            "watchman-token": "bar",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 403)


class TestWatchmanNoTokens(unittest.TestCase):
    def setUp(self):
        self.client = Client()
        watchman_settings.WATCHMAN_TOKEN = None
        watchman_settings.WATCHMAN_TOKENS = None

    def test_200_ok_if_no_token_set(self):
        response = self.client.get(reverse("status"))
        self.assertEqual(response.status_code, 200)

    def test_200_ok_if_no_token_set_but_passed_in(self):
        data = {
            "watchman-token": "foo",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 200)


class TestWatchmanSingleToken(unittest.TestCase):
    def setUp(self):
        self.client = Client()
        self.logger = logging.getLogger("watchman")
        watchman_settings.WATCHMAN_TOKEN = "foo"
        watchman_settings.WATCHMAN_TOKENS = None

    def test_200_ok_if_tokens_match(self):
        data = {
            "watchman-token": "foo",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 200)

    def test_required_token_param_can_be_renamed(self):
        watchman_settings.WATCHMAN_TOKEN_NAME = "custom-token"
        data = {
            "custom-token": "foo",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 200)
        watchman_settings.WATCHMAN_TOKEN_NAME = "watchman-token"

    def test_403_raised_if_missing_token(self):
        response = self.client.get(reverse("status"))
        self.assertEqual(response.status_code, 403)

    def test_403_raised_if_invalid_token(self):
        data = {
            "watchman-token": "bar",
        }
        response = self.client.get(reverse("status"), data)
        self.assertEqual(response.status_code, 403)

    # We cannot easily mock a stacktrace, since:
    #
    #   1. it is not a property of the exception,
    #   2. file paths will change between systems
    #
    # Instead, assert all other response parameters are correct.
    def test_exception_caught_if_no_arguments(self):
        def test_func():
            raise FakeException("test error")

        with (
            mock.patch.object(self.logger, "exception") as mock_exception,
            mock.patch.object(self.logger, "debug") as mock_debug,
        ):
            decorated_func = check(test_func)
            keys = ["ok", "error", "stacktrace"]
            response = decorated_func()
            self.assertIsInstance(response, dict)
            for key in keys:
                self.assertIn(key, response)
            self.assertEqual(response["ok"], False)
            self.assertEqual(response["error"], "test error")
            self.assertIsInstance(response["stacktrace"], str)
            mock_debug.assert_called_once_with("Checking '%s'", "test_func")
            mock_exception.assert_called_once_with(
                "Error calling '%s': %s", "test_func", "test error"
            )

    def test_exception_caught_with_argument(self):
        def test_func(arg):
            raise FakeException("test error")

        with (
            mock.patch.object(self.logger, "exception") as mock_exception,
            mock.patch.object(self.logger, "debug") as mock_debug,
        ):
            decorated_func = check(test_func)
            keys = ["ok", "error", "stacktrace"]
            result = decorated_func("default")
            self.assertIsInstance(result, dict)
            response = cast(dict[str, CheckStatus], result)
            for key in keys:
                self.assertIn(key, response["default"])
            self.assertEqual(response["default"]["ok"], False)
            self.assertEqual(response["default"]["error"], "test error")
            self.assertIsInstance(response["default"]["stacktrace"], str)
            mock_debug.assert_called_once_with(
                "Checking '%s' for '%s'", "test_func", "default"
            )
            mock_exception.assert_called_once_with(
                "Error calling '%s' for '%s': %s",
                "test_func",
                "default",
                "test error",
            )

    def tearDown(self):
        watchman_settings.WATCHMAN_TOKEN = None



import json
from argparse import ArgumentParser
from typing import Any

from django.core.management.base import BaseCommand, CommandError

from watchman.utils import get_checks


class Command(BaseCommand):
    help = "Runs the default django-watchman checks"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "-c",
            "--checks",
            dest="checks",
            help="A comma-separated list of watchman checks to run (full python dotted paths)",
        )
        parser.add_argument(
            "-s",
            "--skips",
            dest="skips",
            help="A comma-separated list of watchman checks to skip (full python dotted paths)",
        )

    def handle(self, *args: Any, **options: Any) -> None:
        check_list: list[str] | None = None
        skip_list: list[str] | None = None

        checks: str | None = options["checks"]
        skips: str | None = options["skips"]

        if checks is not None:
            check_list = checks.split(",")

        if skips is not None:
            skip_list = skips.split(",")

        for check in get_checks(check_list=check_list, skip_list=skip_list):
            if callable(check):
                resp = json.dumps(check())
                if '"ok": false' in resp:
                    raise CommandError(resp)
                elif int(options["verbosity"]) >= 2:
                    self.stdout.write(resp)



from django.urls import re_path

from watchman import views

urlpatterns = [
    re_path(r"^$", views.status, name="status"),
    re_path(r"^dashboard/$", views.dashboard, name="dashboard"),
    re_path(r"^ping/$", views.ping, name="ping"),
]



"""
test_django-watchman
------------

Tests for `django-watchman` utils module.
"""

import unittest
from unittest.mock import patch

from watchman.utils import get_cache, get_checks


class TestWatchman(unittest.TestCase):
    def assert_lists_equal(self, list1, list2):
        self.assertCountEqual(list1, list2)

    @patch("watchman.utils.django_cache.caches", spec_set=dict)
    def test_get_cache_django_17_or_greater(self, get_cache_mock):
        get_cache("foo")
        get_cache_mock.__getitem__.assert_called_once_with("foo")

    def test_get_checks_returns_all_available_checks_by_default(self):
        checks = [check.__name__ for check in get_checks()]
        expected_checks = ["caches", "databases", "storage"]
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_check_list_returns_union(self):
        check_list = ["watchman.checks.caches"]
        checks = [check.__name__ for check in get_checks(check_list=check_list)]
        expected_checks = ["caches"]
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_skip_list_returns_difference(self):
        skip_list = ["watchman.checks.caches"]
        checks = [check.__name__ for check in get_checks(skip_list=skip_list)]
        expected_checks = ["databases", "storage"]
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_matching_check_and_skip_list_returns_empty_list(self):
        check_list, skip_list = ["watchman.checks.caches"], ["watchman.checks.caches"]
        checks = [
            check.__name__
            for check in get_checks(check_list=check_list, skip_list=skip_list)
        ]
        expected_checks = []
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_check_and_skip_list(self):
        check_list = ["watchman.checks.caches", "watchman.checks.databases"]
        skip_list = ["watchman.checks.caches"]
        checks = [
            check.__name__
            for check in get_checks(check_list=check_list, skip_list=skip_list)
        ]
        expected_checks = ["databases"]
        self.assert_lists_equal(checks, expected_checks)

    def test_get_checks_with_paid_checks_disabled_returns_expected_checks(self):
        expected_checks = ["caches", "databases", "storage"]
        checks = [check.__name__ for check in get_checks()]
        self.assert_lists_equal(checks, expected_checks)

    @unittest.skip("Unsure how to test w/ modified settings")
    def test_get_checks_with_paid_checks_enabled_returns_expected_checks(self):
        pass



from typing import TypedDict


class _CheckStatusRequired(TypedDict):
    ok: bool


class CheckStatus(_CheckStatusRequired, total=False):
    """Result of a single health check -- success or error.

    The ``ok`` field is always present. The ``error`` and ``stacktrace``
    fields are only present when the check failed.
    """

    error: str
    stacktrace: str


# A check result: either a direct status (email, storage) or a mapping of
# named statuses keyed by resource name (caches, databases).
CheckResult = CheckStatus | dict[str, CheckStatus]



"""
test_django-watchman
------------

Tests for `django-watchman` management commands.
"""

import unittest
from io import StringIO

from django.core.management import call_command


class TestWatchman(unittest.TestCase):
    def test_successful_management_command_outputs_nothing(self):
        out = StringIO()
        call_command("watchman", stdout=out)
        self.assertEqual("", out.getvalue())

    def test_successful_management_command_outputs_check_status_with_verbosity_2(self):
        out = StringIO()
        call_command("watchman", stdout=out, verbosity="2")
        self.assertIn("caches", out.getvalue())

    def test_successful_management_command_outputs_check_status_with_verbosity_3(self):
        out = StringIO()
        call_command("watchman", stdout=out, verbosity="3")
        self.assertIn("caches", out.getvalue())

    def test_successful_management_command_supports_check_list(self):
        out = StringIO()
        call_command(
            "watchman", stdout=out, checks="watchman.checks.caches", verbosity="3"
        )
        self.assertIn("caches", out.getvalue())
        self.assertNotIn("databases", out.getvalue())

    def test_successful_management_command_supports_skip_list(self):
        out = StringIO()
        call_command(
            "watchman", stdout=out, skips="watchman.checks.email", verbosity="3"
        )
        self.assertIn("caches", out.getvalue())
        self.assertIn("databases", out.getvalue())
        self.assertNotIn("email", out.getvalue())



"""
Test settings for django-watchman tests.
"""

from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

DEBUG = True
USE_TZ = True

SECRET_KEY = "test-secret-key-for-django-watchman"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    },
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
    },
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

ROOT_URLCONF = "watchman.urls"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "watchman",
]

SITE_ID = 1

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# Media files for storage checks
MEDIA_ROOT = BASE_DIR / ".test_media"
MEDIA_URL = "/media/"

# Email backend for testing
EMAIL_BACKEND = "django.core.mail.backends.locmem.EmailBackend"



import logging
import traceback
from collections.abc import Callable
from functools import wraps
from typing import Any

from django.http import HttpRequest, HttpResponse, HttpResponseForbidden
from django.views.decorators.csrf import csrf_exempt

from watchman import settings
from watchman.types import CheckResult, CheckStatus

logger: logging.Logger = logging.getLogger("watchman")


def check(func: Callable[..., CheckResult]) -> Callable[..., CheckResult]:
    """
    Decorator which wraps checks and returns an error response on failure.
    """

    def wrapped(*args: Any, **kwargs: Any) -> CheckResult:
        check_name = getattr(func, "__name__", repr(func))
        arg_name = None
        if args:
            arg_name = args[0]
        try:
            if arg_name:
                logger.debug("Checking '%s' for '%s'", check_name, arg_name)
            else:
                logger.debug("Checking '%s'", check_name)
            response: CheckResult = func(*args, **kwargs)
        except Exception as e:
            message = str(e)
            error_status: CheckStatus = {
                "ok": False,
                "error": message,
                "stacktrace": traceback.format_exc(),
            }
            # The check contains several individual checks (e.g., one per
            # database). Preface the results by name.
            if arg_name:
                named_result: dict[str, CheckStatus] = {arg_name: error_status}
                response = named_result
                logger.exception(
                    "Error calling '%s' for '%s': %s", check_name, arg_name, message
                )
            else:
                response = error_status
                logger.exception("Error calling '%s': %s", check_name, message)

        return response

    return wrapped


def parse_auth_header(auth_header: str) -> str:
    """Parse the ``Authorization`` header and return the token value.

    Expected format: ``WATCHMAN-TOKEN Token="ABC123"``

    Raises :exc:`KeyError` when no ``Token`` parameter is found.
    """

    for part in auth_header.split():
        key, sep, value = part.partition("=")
        if sep and key == "Token":
            return value.strip('"')
    raise KeyError("Token")


def token_required(
    view_func: Callable[..., HttpResponse],
) -> Callable[..., HttpResponse]:
    """
    Decorator which ensures that one of the WATCHMAN_TOKENS is provided if set.

    WATCHMAN_TOKEN_NAME can also be set if the token GET parameter must be
    customized.

    """

    def _get_passed_token(request: HttpRequest) -> str | None:
        """
        Try to get the passed token, starting with the header and fall back to `GET` param
        """

        try:
            auth_header = request.META["HTTP_AUTHORIZATION"]
            token = parse_auth_header(auth_header)
        except KeyError:
            token = request.GET.get(settings.WATCHMAN_TOKEN_NAME)
        return token

    def _validate_token(request: HttpRequest) -> bool:
        if settings.WATCHMAN_TOKENS:
            watchman_tokens = settings.WATCHMAN_TOKENS.split(",")
        elif settings.WATCHMAN_TOKEN:
            watchman_tokens = [
                settings.WATCHMAN_TOKEN,
            ]
        else:
            return True

        return _get_passed_token(request) in watchman_tokens

    @csrf_exempt
    @wraps(view_func)
    def _wrapped_view(request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        if _validate_token(request):
            return view_func(request, *args, **kwargs)

        return HttpResponseForbidden()

    return _wrapped_view


if settings.WATCHMAN_AUTH_DECORATOR is None:

    def auth(view_func: Callable[..., HttpResponse]) -> Callable[..., HttpResponse]:
        @csrf_exempt
        @wraps(view_func)
        def _wrapped_view(
            request: HttpRequest, *args: Any, **kwargs: Any
        ) -> HttpResponse:
            return view_func(request, *args, **kwargs)

        return _wrapped_view

elif settings.WATCHMAN_AUTH_DECORATOR == "watchman.decorators.token_required":
    # Avoid import loops
    auth = token_required
else:
    from importlib import import_module

    mod_name, dec = settings.WATCHMAN_AUTH_DECORATOR.rsplit(".", 1)
    auth = getattr(import_module(mod_name), dec)



import warnings
from typing import Any

from django.db.transaction import non_atomic_requests
from django.http import HttpRequest, HttpResponse, JsonResponse
from django.shortcuts import render
from django.utils.translation import gettext as _

from watchman import __version__, settings
from watchman.decorators import auth
from watchman.utils import get_checks

WATCHMAN_VERSION_HEADER: str = "X-Watchman-Version"


def _get_check_params(
    request: HttpRequest,
) -> tuple[list[str] | None, list[str] | None]:
    check_list: list[str] | None = None
    skip_list: list[str] | None = None

    if len(request.GET) > 0:
        if "check" in request.GET:
            check_list = request.GET.getlist("check")
        if "skip" in request.GET:
            skip_list = request.GET.getlist("skip")

    return (check_list, skip_list)


def _deprecation_warnings() -> None:
    if settings.WATCHMAN_TOKEN:
        warnings.warn(
            "`WATCHMAN_TOKEN` setting is deprecated, use `WATCHMAN_TOKENS` instead. It will be removed in django-watchman 1.0",
            DeprecationWarning,
            stacklevel=2,
        )


def _disable_apm() -> None:
    # New Relic
    try:
        import newrelic.agent

        newrelic.agent.ignore_transaction(flag=True)
    except ImportError:
        pass
    # Datadog
    try:
        from ddtrace import tracer
        from ddtrace.constants import MANUAL_DROP_KEY

        tracer.current_span().set_tag(MANUAL_DROP_KEY)
    except (AttributeError, ImportError):
        pass


def run_checks(request: HttpRequest) -> tuple[dict[str, Any], bool]:
    _deprecation_warnings()

    if settings.WATCHMAN_DISABLE_APM:
        _disable_apm()

    checks: dict[str, Any] = {}
    ok: bool = True

    check_list, skip_list = _get_check_params(request)

    for check in get_checks(check_list=check_list, skip_list=skip_list):
        if callable(check):
            _check = check()
            # Set our HTTP status code if there were any errors
            if settings.WATCHMAN_ERROR_CODE != 200:
                for _type in _check:
                    if isinstance(_check[_type], dict):
                        result = _check[_type]
                        if not result["ok"]:
                            ok = False
                    elif isinstance(_check[_type], list):
                        for entry in _check[_type]:
                            for result in entry:
                                if not entry[result]["ok"]:
                                    ok = False
            checks.update(_check)

    return checks, ok


@auth
@non_atomic_requests
def status(request: HttpRequest) -> HttpResponse:
    checks, ok = run_checks(request)

    if not checks:
        response = JsonResponse(
            {
                "error": 404,
                "message": _("No checks found"),
            },
            status=404,
        )
    else:
        http_code = 200 if ok else settings.WATCHMAN_ERROR_CODE
        response = JsonResponse(checks, status=http_code)

    if settings.EXPOSE_WATCHMAN_VERSION:
        response[WATCHMAN_VERSION_HEADER] = __version__

    return response


@non_atomic_requests
def bare_status(request: HttpRequest) -> HttpResponse:
    checks, ok = run_checks(request)
    http_code = 200 if ok else settings.WATCHMAN_ERROR_CODE
    return HttpResponse(status=http_code, content_type="text/plain")


def ping(request: HttpRequest) -> HttpResponse:
    if settings.WATCHMAN_DISABLE_APM:
        _disable_apm()
    return HttpResponse("pong", content_type="text/plain")


@auth
@non_atomic_requests
def dashboard(request: HttpRequest) -> HttpResponse:
    checks, overall_status = run_checks(request)

    expanded_checks: dict[str, Any] = {}
    for key, value in checks.items():
        if isinstance(value, dict):
            # For some systems (eg: email, storage) value is a
            # dictionary of status
            #
            # Example:
            # {
            #     'ok': True,  # Status
            # }
            #
            # Example:
            # {
            #     'ok': False,  # Status
            #     'error': "RuntimeError",
            #     'stacktrace': "...",
            # }
            single_status = value.copy()
            single_status["name"] = ""
            expanded_check: dict[str, Any] = {
                "ok": value["ok"],
                "statuses": [single_status],
            }
        else:
            # For other systems (eg: cache, database) value is a
            # list of dictionaries of dictionaries of statuses
            #
            # Example:
            # [
            #     {
            #         'default': {  # Cache/database name
            #             'ok': True,  # Status
            #         }
            #     },
            #     {
            #         'non-default': {  # Cache/database name
            #             'ok': False,  # Status
            #             'error': "RuntimeError",
            #             'stacktrace': "...",
            #         }
            #     },
            # ]
            statuses: list[dict[str, Any]] = []
            for outer_status in value:
                for name, inner_status in outer_status.items():
                    detail = inner_status.copy()
                    detail["name"] = name
                    statuses.append(detail)

            expanded_check = {
                "ok": all(detail["ok"] for detail in statuses),
                "statuses": statuses,
            }
        expanded_checks[key] = expanded_check

    response = render(
        request,
        "watchman/dashboard.html",
        {
            "checks": expanded_checks,
            "overall_status": overall_status,
            "watchman_version": __version__,
            "expose_watchman_version": settings.EXPOSE_WATCHMAN_VERSION,
        },
    )

    if settings.EXPOSE_WATCHMAN_VERSION:
        response[WATCHMAN_VERSION_HEADER] = __version__

    return response



import uuid
from pathlib import PurePath
from typing import Any

from django.core.files.base import ContentFile
from django.core.files.storage import default_storage
from django.core.mail import EmailMessage
from django.db import connections

from watchman import settings as watchman_settings
from watchman import utils
from watchman.decorators import check
from watchman.types import CheckResult, CheckStatus


def _check_caches(caches: dict[str, Any]) -> list[CheckResult]:
    return [_check_cache(cache) for cache in sorted(caches)]


@check
def _check_cache(cache_name: str) -> dict[str, CheckStatus]:
    key = f"django-watchman-{uuid.uuid4()}"
    value = f"django-watchman-{uuid.uuid4()}"

    cache = utils.get_cache(cache_name)

    cache.set(key, value)
    cache.get(key)
    cache.delete(key)
    return {cache_name: {"ok": True}}


def _check_databases(databases: dict[str, Any]) -> list[CheckResult]:
    return [_check_database(database) for database in sorted(databases)]


@check
def _check_database(database: str) -> dict[str, CheckStatus]:
    connection = connections[database]
    with connection.cursor() as cursor:
        cursor.execute("SELECT 1")
    return {database: {"ok": True}}


@check
def _check_email() -> CheckStatus:
    headers = {"X-DJANGO-WATCHMAN": "true"}
    headers.update(watchman_settings.WATCHMAN_EMAIL_HEADERS)
    email = EmailMessage(
        "django-watchman email check",
        "This is an automated test of the email system.",
        watchman_settings.WATCHMAN_EMAIL_SENDER,
        watchman_settings.WATCHMAN_EMAIL_RECIPIENTS,
        headers=headers,
    )
    email.send()
    return {"ok": True}


@check
def _check_storage() -> CheckStatus:
    # Use relative path within storage - Django handles the base location
    storage_subdir = watchman_settings.WATCHMAN_STORAGE_PATH
    # Convert absolute paths to empty string (use storage root)
    if storage_subdir and PurePath(storage_subdir).is_absolute():
        storage_subdir = ""
    filename = f"django-watchman-{uuid.uuid4()}.txt"
    if storage_subdir:
        filename = str(PurePath(storage_subdir) / filename)
    content = b"django-watchman test file"
    path = default_storage.save(filename, ContentFile(content))
    default_storage.size(path)
    default_storage.open(path).read()
    default_storage.delete(path)
    return {"ok": True}


def caches() -> dict[str, list[CheckResult]]:
    return {"caches": _check_caches(watchman_settings.WATCHMAN_CACHES)}


def databases() -> dict[str, list[CheckResult]]:
    return {"databases": _check_databases(watchman_settings.WATCHMAN_DATABASES)}


def email() -> dict[str, CheckResult]:
    return {"email": _check_email()}


def storage() -> dict[str, CheckResult]:
    return {"storage": _check_storage()}



from typing import Any

from django.conf import settings

from watchman.constants import DEFAULT_CHECKS, PAID_CHECKS

# TODO: these should not be module level (https://github.com/mwarkentin/django-watchman/issues/13)
WATCHMAN_ENABLE_PAID_CHECKS: bool = getattr(
    settings, "WATCHMAN_ENABLE_PAID_CHECKS", False
)
WATCHMAN_AUTH_DECORATOR: str | None = getattr(
    settings, "WATCHMAN_AUTH_DECORATOR", "watchman.decorators.token_required"
)
# TODO: Remove for django-watchman 1.0
WATCHMAN_TOKEN: str | None = getattr(settings, "WATCHMAN_TOKEN", None)
WATCHMAN_TOKENS: str | None = getattr(settings, "WATCHMAN_TOKENS", None)
WATCHMAN_TOKEN_NAME: str = getattr(settings, "WATCHMAN_TOKEN_NAME", "watchman-token")
WATCHMAN_ERROR_CODE: int = getattr(settings, "WATCHMAN_ERROR_CODE", 500)
WATCHMAN_EMAIL_SENDER: str = getattr(
    settings, "WATCHMAN_EMAIL_SENDER", "watchman@example.com"
)
WATCHMAN_EMAIL_RECIPIENTS: list[str] = getattr(
    settings, "WATCHMAN_EMAIL_RECIPIENTS", ["to@example.com"]
)
WATCHMAN_EMAIL_HEADERS: dict[str, str] = getattr(settings, "WATCHMAN_EMAIL_HEADERS", {})

WATCHMAN_CACHES: dict[str, Any] = getattr(settings, "WATCHMAN_CACHES", settings.CACHES)
WATCHMAN_DATABASES: dict[str, Any] = getattr(
    settings, "WATCHMAN_DATABASES", settings.DATABASES
)

WATCHMAN_DISABLE_APM: bool = getattr(settings, "WATCHMAN_DISABLE_APM", False)

WATCHMAN_STORAGE_PATH: str = getattr(
    settings, "WATCHMAN_STORAGE_PATH", settings.MEDIA_ROOT
)

_checks: tuple[str, ...] = DEFAULT_CHECKS

if WATCHMAN_ENABLE_PAID_CHECKS:
    _checks = DEFAULT_CHECKS + PAID_CHECKS

WATCHMAN_CHECKS: tuple[str, ...] = getattr(settings, "WATCHMAN_CHECKS", _checks)

EXPOSE_WATCHMAN_VERSION: bool = getattr(settings, "EXPOSE_WATCHMAN_VERSION", False)
